﻿#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2012 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using System.Collections.Generic;
using HeuristicLab.Encodings.IntegerVectorEncoding;

namespace HeuristicLab.Problems.GeneralizedQuadraticAssignment.Common {
  public class IntegerVectorEqualityComparer : EqualityComparer<IntegerVector> {
    public override bool Equals(IntegerVector x, IntegerVector y) {
      if (x == null || y == null) return false;
      if (x.Length != y.Length) return false;

      for (int i = 0; i < x.Length; i++)
        if (x[i] != y[i]) return false;

      return true;
    }

    public override int GetHashCode(IntegerVector obj) {
      return obj.GetHashCode();
    }

    public static double GetSimilarity(IntegerVector a, IntegerVector b) {
      int similar = 0;
      for (int i = 0; i < a.Length; i++) {
        if (a[i] == b[i]) similar++;
      }
      return similar / (double)a.Length;
    }

    public static double GetDistance(IntegerVector a, IntegerVector b) {
      return 1.0 - GetSimilarity(a, b);
    }

    public static IEnumerable<int> GetDifferingIndices(IntegerVector a, IntegerVector b) {
      for (int i = 0; i < a.Length; i++)
        if (a[i] != b[i]) yield return i;
    }
  }
}
